/**
  ******************************************************************************
  * @file           : main.c
  * @author         ：SINOMCU-AE
  * @brief          : Main program body
  *    
  *          This file provides example code for EN CSS,HSE,LSE,MCO out and printf by uart.
  * RCC CSS,HSE ready PLL 24M for system,HSE Error HSI to PLL 24M for system.IWDT set 1S,
  * if IWDT reset LED2 Toggle.LED1 blink by sysTick soft timer,HSE period 1S,HSI period 250mS;
  * uart send information of running count(redirect printf).For details, see "readme.txt"   
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, SinoMCU shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

#define TRY_CHECK_HSE        1         // 1:try to check HSE OK,0:after HSI Error not try to check HSE

// LED blink half cycle in ms  
#define LED_BLINK_HALF_PER   500       // unit ms
FlagStatus RCC_CSS_Flag      = RESET;  // when HSE Error,CSSF set 
#if(defined(TRY_CHECK_HSE) && TRY_CHECK_HSE) 
FlagStatus CHECK_HSE_Flag    = RESET;  // when SET,need check HSE ready
#endif
/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) {
  uint32_t count=0;
  uint32_t reset_flag =0;

  reset_flag= MS32_RCC_GetResetFlag(ENABLE);               // get reset source and clear
  sysclk_init();                       // sysclk 24M,HSE 8M /1*3;LSE en,MOC out LSE/2
  SystemCoreClockUpdate();             // get System Core Clock
    
  sys_tick_init();
  gpio_init();
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);       // uart clk HSI 8M     
  usart1_uart_init(115200);
#endif
  if(reset_flag & MS32_RCC_FLAG_IWDGRSTF) {                          // IWDT reset occur
    LED2_TOGGLE(); 
    printf("\r\n!!Inf:!!!Reset source include IWDT!");  
  }
  
  MS32_RCC_MCOConfig(MS32_RCC_MCO_SOURCE_PLL, MS32_RCC_MCO_DIV2);    // had finish pin config
  iwdg_init();
  LED1_ON(); 
  
  printf("\r\n\r\n*****HSE LSE MCO CSS Example*****");
  if( MS32_RCC_PLL_GetMainSource() == MS32_RCC_PLLSOURCE_HSE) {      // HSE to PLL
    printf("\r\n--Inf:HSE to PLL,PLL out for system clock.");
  } else if ( MS32_RCC_PLL_GetMainSource() == MS32_RCC_PLLSOURCE_HSI_DIV_2) {  // HSI to PLL
    printf("\r\n!-Inf:HSI to PLL,PLL out for system clock.");
#if(defined(TRY_CHECK_HSE) && TRY_CHECK_HSE)
    CHECK_HSE_Flag = SET;
#endif
  } else {
    printf("\r\n!!Inf:HSI for system clock.");                       // use HSI
#if(defined(TRY_CHECK_HSE) && TRY_CHECK_HSE) 
    CHECK_HSE_Flag = SET;
#endif  
  }
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);
  
  Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);
  while(1) {
    RCC_CSS_Dispose();                                               // CSS application function

    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      if( MS32_RCC_PLL_GetMainSource() == MS32_RCC_PLLSOURCE_HSE) {  // HSE to PLL
        Start_User_MS_SF_Timer(LED_BLINK_HALF_PER);                  // clear soft timer ov
      } else {
        Start_User_MS_SF_Timer(LED_BLINK_HALF_PER >>2);              // clear soft timer ov        
      }
      LED1_TOGGLE();
      count++;
      printf("\r\n--Inf: running count:%d",count);
      MS32_IWDG_Refresh();
    }
  }
}

#define HSE_READY_TIME        (96000)       // = 2000us / (1 / 48MHz)
/**
  * @brief  when CSSF set use HSI to PLL,try to HSE,when HSE ready,HSE to PLL 
  *         eg. HSI to PLL out 24M,HSE to PLL out 24M.
  * @param  None
  * @retval None
  */
void RCC_CSS_Dispose(void) {
#if(defined(TRY_CHECK_HSE) && TRY_CHECK_HSE) 
  uint32_t    wait_time;
#endif
  if( RCC_CSS_Flag ==SET ) {
    RCC_CSS_Flag = RESET;
    printf("\r\n##ERROR:HSE Error!Use HSI for system.");
    // when HSE to PLL and PLL out for sytem,HSE Error,PLL off;
    MS32_RCC_PLL_ConfigDomain_SYS(MS32_RCC_PLLSOURCE_HSI_DIV_2, MS32_RCC_PLL_MUL_6);     // HSI/2=8M/2=4M,PLL 24M
    MS32_RCC_PLL_Enable();
    while(MS32_RCC_PLL_IsReady() != 1){ ; }      // add timeout will be better
    MS32_RCC_SetSysClkSource(MS32_RCC_SYS_CLKSOURCE_PLL);  // HSE Error,change to HSI,need select PLL again
#if(defined(TRY_CHECK_HSE) && TRY_CHECK_HSE) 
    CHECK_HSE_Flag =SET;
  }
  if(CHECK_HSE_Flag ==SET) {
    MS32_RCC_HSE_EnableCSS();                    // HSE Error clear CSSON
    MS32_RCC_HSE_Enable();
    wait_time = HSE_READY_TIME;
    while (!MS32_RCC_HSE_IsReady() && (--wait_time));
    if(MS32_RCC_HSE_IsReady() == SET) {          // HSE ready
      CHECK_HSE_Flag =RESET;
      MS32_RCC_SetSysClkSource(MS32_RCC_SYS_CLKSOURCE_HSI);// for disable PLL
      MS32_RCC_PLL_Disable();                    // for change PLL setting          
      MS32_RCC_PLL_ConfigDomain_SYS(MS32_RCC_PLLSOURCE_HSE_DIV_1, MS32_RCC_PLL_MUL_3);   // PLL 24M
      MS32_RCC_PLL_Enable();
      while(MS32_RCC_PLL_IsReady() != 1){ ; }    // add timeout will be better
      MS32_RCC_SetSysClkSource(MS32_RCC_SYS_CLKSOURCE_PLL);
      printf("\r\n--Inf:HSE to PLL for system.");        
    }
#endif
  }
}

/**
  * @brief  call by NMI_Handler,CSSF set(HSE Error)
  * @param  None
  * @retval None
  */
void RCC_CSS_CallBack(void) {
  RCC_CSS_Flag =SET;
}

/**
  * @brief  SystemClock_Config HSE to PLL*3 sysclk 24M,HSI ON,LSE ON
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  MS32_RCC_HSE_EnableCSS();                      // CSSON set before HSE ON   
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_ON;
  osc_cfg.LSEState = MS32_RCC_LSE_ON_DRV_H;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  if( MS32_RCC_HSE_IsReady() ) {
    pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSE; // HSE*3 24M  
    pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
    pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL3;
  } else {
    pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;   
    pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL6;       // HSI/2=4M,*6=24M
  }
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  if( MS32_RCC_PLL_IsReady() ) {
    clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  }  else {
    clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_HSI;
  }    

  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}
/******************************** END OF FILE *********************************/
